from netgen.geom2d import *
from ngsolve import *

geo = SplineGeometry()
geo.AddRectangle((0,0),(2,2),
    bcs=["b","r","t","l"],
    leftdomain=1)
geo.AddRectangle((0.5,0.5),(1.5,1.5),
    bcs=["b2","r2","t2","l2"],
    leftdomain=2, rightdomain=1)

geo.SetMaterial(1,'outer')
geo.SetMaterial(2,'inner')

mesh = Mesh(geo.GenerateMesh(maxh=2))

condense = True
# HDG on inner domain
fes_u = L2(mesh, order = 0)#, definedon= mesh.Materials("inner"))
fes_q = VectorL2(mesh, order = 0)#, definedon= mesh.Materials("inner"))
fes_lam = FacetFESpace(mesh, order = 0, definedon=mesh.Materials("inner"))
fes = FESpace([fes_u, fes_q, fes_lam])

(u,q, uhat), (v, r, vhat) = fes.TnT()

a = BilinearForm(fes, condense=condense)

h = specialcf.mesh_size
n = specialcf.normal(mesh.dim)

inner_func = CoefficientFunction([1 if mat=='inner' else 0 for mat in 
  mesh.GetMaterials()])
a +=  (q*r+u*v-(-u*div(r)-q*grad(v)))*dx("inner")
qhatn = q*n-(u-uhat)
a +=  -inner_func*(uhat*r*n+qhatn*(v-vhat))*dx(element_boundary=True)


a.Assemble()

for el in fes.Elements(VOL):
    if el.mat == "outer" : # mark off dofs
        for j in el.dofs:
          fes.FreeDofs()[j] = False
          fes.FreeDofs(True)[j] = False
          fes.SetCouplingType(j, COUPLING_TYPE.UNUSED_DOF)

#a.Assemble() # this line fail

inva = a.mat.Inverse(fes.FreeDofs(condense))
f = LinearForm(fes)
f += v*dx
f.Assemble()

gfu = GridFunction(fes)
if condense :
  f.vec.data += a.harmonic_extension_trans*f.vec
gfu.vec.data = inva*f.vec
if condense :
  gfu.vec.data += a.inner_solve*f.vec
  gfu.vec.data += a.harmonic_extension*gfu.vec

print(gfu.vec)
